#!/usr/bin/python

#######################################################################
# $Id: pytego.py,v 1.36 2003/06/25 15:38:35 churchil Exp $
#######################################################################


import os,sys,getopt
import pygame
from pygame.locals import *

import socket
from select import select
import globals
import action
from action import Action
import constants
from connection import Connection
from connection_poller_thread import ConnectionPollerThread


# -------------------------------------------
# F U N C T I O N S
# -------------------------------------------
def s2g((x,y)):
	"""
	trivial function to convert screen coordinates
	to game coordinates
	"""
	return (x/globals.WIDTH),(y/globals.HEIGHT)

def g2s((x,y)):
	"""
	trivial function to convert game coordinates
	to screen coordinates; screen coordinates are
	all relative to the topleft corner :)
	"""
	return (x*globals.WIDTH),(y*globals.HEIGHT)

def load_image(name, dir='.', colorkey=None):
	fullname = os.path.join('data',dir,name)
	try:
		image = pygame.image.load(fullname)
	except pygame.error, message:
		print 'Cannot load image: ', name
		raise SystemExit, message
	image = image.convert()
	if colorkey is not None:
		if colorkey is -1:
			colorkey = image.get_at((0,0))
		image.set_colorkey(colorkey, RLEACCEL)
	return image, image.get_rect()

def printUsage():
	""" seperate options with 2 tabs """
	print "\n \
Usage: pytego [options] \n \
\n \
-h, --help		Print this text and exit\n \
-s, --server		Run as a server\n \
-H, --host <host>	Connect to host (Default: localhost) \n \
-p, --port <port>	Use other port (Default: 9998) \n \
-C, --color {red|blue}	Play as 'red' or 'blue' (Default: red) \n \
-f, --fullscreen	Run fullscreen. \n \
-r, --resolution 	{640x480|800x600|1024x768|1280x1024}		\n \
		"
			
def checkNetEvents(connection):
	try:
		line = connection.readLine()
	except IOError:
		return 0

	if line:
		p = line.split()
		action = int(p[0])
		try:
			unit = Unit(p[1],p[2],(int(p[3]),int(p[4])))
			x = int(p[5])
			y = int(p[6])
			a = Action(action,unit,x,y)
			globals.action_queue.append(a)
			return 1
		except IndexError:
			globals.action_queue.append(Action(action))
	return 0

def sendAction(action, connection):
    try:
        connection.send(action.toString()+'\n')
    except IOError:
        print 'error sending action'

def doDone(connection):
	globals.myturn = 0
	sendAction(Action(action.DONE), connection)

def handleBoardEvent(board, clickedArea, connection, selectedUnit, (targetx, targety)):
	if globals.playing == constants.GAME_PLAYING and globals.myturn:
		turnComplete = 0
		""" the user selected something (selectedUnit), then clicked somewhere (x,y) """
		target = board.unitAt((targetx, targety))
		if target is not None:
			""" strike """
			outcome = selectedUnit.strike(target)
			if outcome == constants.STRIKE_INVALID:
				globals.textArea.addText("invalid attack")
			if outcome == constants.STRIKE_FAILED or outcome == constants.STRIKE_SAME_UNIT:
				removeAction = Action(action.REMOVE, selectedUnit)
				globals.action_queue.append(removeAction)
				sendAction(removeAction, connection)
				turnComplete = 1
			if outcome == constants.STRIKE_SUCCESSFUL or outcome == constants.STRIKE_SAME_UNIT:
				removeAction = Action(action.REMOVE, target)
				globals.action_queue.append(removeAction)
				sendAction(removeAction, connection)
				# if this is a successful strike, also move
				if outcome != constants.STRIKE_SAME_UNIT:
					moveAction = Action(action.MOVE, selectedUnit, targetx, targety)
					globals.action_queue.append(moveAction)
					sendAction(moveAction, connection)
				turnComplete = 1
			if outcome == constants.STRIKE_HIT_FLAG:
				sendAction(Action(action.VICTORY), connection)
				globals.textArea.addText("Game Over! Congratulations, you were victorius")
				globals.playing = constants.GAME_ENDED
				turnComplete = 1 # ???
		else:
			if selectedUnit is not None and selectedUnit.canMove((targetx, targety)):
				terrain = board.background.terrainAt((targetx, targety))
				if terrain is not None and terrain == constants.TERRAIN_NORMAL:
					moveAction = Action(action.MOVE, selectedUnit, targetx, targety)
					globals.action_queue.append(moveAction)
					sendAction(moveAction, connection)
					turnComplete = 1

		''' did you actually do something? '''
		if turnComplete:
			doDone(connection)
	else:
		if globals.playing != constants.GAME_PLAYING:
			if globals.playing == constants.GAME_ENDED:
				globals.textArea.addText("game over")
			else:
				globals.textArea.addText("game has not started yet")
		else:
			globals.textArea.addText("please wait your turn")

def handleBoxEvent(box, clickedArea, connection, selectedUnit, (targetx, targety)):
	if globals.playing == constants.GAME_WAITING_TO_START:
		target = clickedArea.unitAt((targetx,targety))
		if target is not None:
			globals.textArea.addText("cannot place unit in occupied square ("+str(targetx)+","+str(targety))
		else:	
			if (globals.color == constants.RED and targety < 6) or (globals.color == constants.BLUE and targety > 3):
				globals.textArea.addText('you cannot place a unit outside your deployment zone(four squares from your edge)')
			else:
				placeAction = Action(action.PLACE, selectedUnit, targetx, targety)
				globals.action_queue.append(placeAction)
				""" do not send over network, DEPLOY will do that """

def handleGameEvent(board, box, connection):
	""" sort of a generic handler, mostly to keep code out of the event-hanlding
	loop. This can be improved if things get too messy or complex """

	""" for now, this only handles K_SPACE event """
	if globals.playing == constants.GAME_WAITING_TO_START:
		if len(board.world) < 40:
			globals.textArea.addText("You must deploy ALL your troops before beginning")
		else:
			for key in board.world.keys():
				unit = board.world[key]
				if unit is not None and unit.color == globals.color:
					deployAction = Action(action.DEPLOY, unit, unit.gx, unit.gy)
					""" do not deal with this action locally, except to set own-deployed """
					globals.own_deployed = 1
					sendAction(deployAction, connection)
			
			globals.textArea.addText("now waiting for other player to deploy")


def updateGameState():
	if globals.playing == constants.GAME_WAITING_TO_START:
		if globals.own_deployed and globals.opponent_deployed:
			globals.textArea.addText("game on")
			globals.playing = constants.GAME_PLAYING
	elif globals.playing == constants.GAME_PLAYING:
		pass
	elif globals.playing == constants.GAME_ENDED:
		pass
# -------------------------------------------
# C L A S S E S
# -------------------------------------------
class StatusArea:
	def __init__(self, rectstyle):
		self.rect = pygame.Rect(rectstyle)
		self.surf = pygame.Surface((self.rect.width, self.rect.height))
		self.font = pygame.font.Font(None, 18)
		self.__aa = 1
		self.__color = (255,255,255)
		self.lineHeight = self.font.get_linesize()
		self.pieces = { constants.BLUE:constants.STARTING_PIECES.copy(), constants.RED:constants.STARTING_PIECES.copy()}
		self.update(None)

	def update(self, unit):
		if unit is not None:
			self.pieces[unit.color][unit.unit] = self.pieces[unit.color][unit.unit] - 1
		""" can I sort this? please do """
		self.surf.fill((0,0,0))
		i = 0
		for k in self.pieces.keys():
			p = self.pieces[k]
			text = self.font.render("--------  "+k+"  --------", self.__aa, self.__color)
			textpos = text.get_rect()
			textpos.centerx = self.surf.get_rect().centerx
			self.surf.blit(text, (textpos.x,i))
			i += self.lineHeight
			for u in p.keys():
				line = u + ": " + str(p[u])
				self.surf.blit(self.font.render(line, self.__aa, self.__color), (4,i))
				i += self.lineHeight

	def getStatus(self):
		return self.surf


class TextArea:
	""" I guess, someday, I could pass in font stuff, for
	now this is sufficient """
	""" do I need a bgcolor? """
	def __init__(self, rectstyle):
		self.rect = pygame.Rect(rectstyle)
		self.font = pygame.font.Font(None, 18)
		self.__aa = 1
		self.__color = (255,255,255)
		self.textSurf = pygame.Surface((self.rect.width, self.rect.height))
		self.line = 0
		self.lineHeight = self.font.get_linesize()
		# self.textSurf.get_rect().inflate(self.rect.width, self.rect.height)

	def getText(self):
		return self.textSurf

	def addText(self, str):
		""" trim if str longer than self.rect.width """
		self.textSurf.blit(self.font.render(str, self.__aa, self.__color), (0,self.line))
		self.line += self.lineHeight
		if self.line >= self.rect.height:
			self.textSurf.fill((0,0,0))
			self.line = 0
		# self.textSurf.get_rect().inflate(self.rect.width, self.rect.height)


class GameArea:
	"""
	keeps track of objects in game areas, right now either Board or Box
	- perhaps add some sort of dimensions and a "inArea" function...

	Coordinates (gx, gy) are relative to upper lef corner of *game area*
	not screen.
	"""
	def __init__(self, rectstyle, handler):
		self.world = {}
		self.selectedUnit = None
		self.sprites = pygame.sprite.RenderPlain()
		self.rect = pygame.Rect(rectstyle)
		self.handler = handler
		""" this is temporary, I will be splitting Board and Box into separate classes
		that are derived from GameArea; this will go *only* in Board class """
		self.background = None

	def addPiece(self,unit):
		self.sprites.add(unit)
		self.world[unit.gx, unit.gy] = unit

	def addArmy(self,army):
		for a in army:
			self.addPiece(a)

	def removePiece(self,unit):
		if unit is not None:
			unit = self.unitAt((unit.gx, unit.gy))
			self.sprites.remove(unit)
			self.world[unit.gx, unit.gy] = None

	def movePiece(self, unit, gx, gy):
		if unit is not None:
			unit = self.unitAt((unit.gx, unit.gy))
			self.world[unit.gx, unit.gy] = None # clear out the old
			unit.move((gx, gy)) # move the new
			self.world[gx, gy] = unit # update the board

	def unitAt(self, (x,y)):
		try:
			return self.world[x, y]
		except KeyError:
			# globals.textArea.addText("no unit at "+ str((x,y)))
			return None
	
	def s2g(self, (x,y)):
		return ((x-self.rect.x)/globals.WIDTH), ((y-self.rect.y)/globals.HEIGHT)

	def handleEvent(self, clickedarea, connection, selectedUnit, (targetx, targety)):
		self.handler(self, clickedarea, connection, selectedUnit, (targetx, targety))

	""" this is just for "debugging" """
	def setName(self, name):
		self.name = name


class Unit(pygame.sprite.Sprite):
	def __init__(self,unit,color,(x,y)):
		pygame.sprite.Sprite.__init__(self)
		self.unit = unit
		self.color = color
		img = str(constants.RANKS[unit])
		if color != globals.color:
			img = 'blank'
		self.image, self.rect = load_image(img+'-'+color+'.png','pieces',-1)
		self.image = pygame.transform.scale(self.image,(globals.WIDTH,globals.HEIGHT))
		self.gx, self.gy = x,y
		self.movepos = g2s((self.gx,self.gy))

	def toString(self):
		return "%s %s %d %d" % (self.unit, self.color, self.gx, self.gy)

	def __str__(self):
		return self.toString()

	def update(self):
		self.rect.topleft = self.movepos

	def canMove(self, (x,y)):
		if self.unit == 'bomb' or self.unit == 'flag':
			return 0
		else:
			return self.__isAdjacent((x,y))

	def move(self, (x,y)):
		self.movepos = g2s((x,y))
		self.gx, self.gy = x,y

	def strike(self, target):
		""" if SHOW_OPPOSING_UNIT property is set """
		globals.textArea.addText("you have attacked a "+target.unit)
		if self.unit == 'flag' or self.unit == 'bomb':
			return constants.STRIKE_INVALID # a bomb or a flag cannot strike!
		if self.__isAdjacent((target.gx, target.gy), 0):
			if target.color == self.color:
				return constants.STRIKE_INVALID
			if target.unit == self.unit:
				return constants.STRIKE_SAME_UNIT
			if target.unit == 'flag':
				return constants.STRIKE_HIT_FLAG
			if target.unit == 'bomb' and self.unit != 'miner':
				return constants.STRIKE_FAILED
			if self.unit == 'spy' and target.unit != 'marshall':
				return constants.STRIKE_FAILED
			if self.unit == 'spy' and target.unit == 'marshall':
				return constants.STRIKE_SUCCESSFUL
			if constants.RANKS[self.unit] < constants.RANKS[target.unit]:
				return constants.STRIKE_SUCCESSFUL
			else:
				return constants.STRIKE_FAILED
		return constants.STRIKE_INVALID

	""" should scouts be able to jump over water??? if not, check lineOfSight """
	def __isAdjacent(self, (x,y), checkScout=1):
		""" checkScout: adjacency, with regards to movement, is not 
		limited to 1 """
		if x == self.gx or y == self.gy:
			if (self.unit == 'scout' and checkScout==1) or (x!=self.gx and abs(x-self.gx)==1) or (y!=self.gy and abs(y-self.gy)==1):
				return 1
			else:
				globals.textArea.addText("can't move/strike more than one square per turn")
		else:
			globals.textArea.addText("can't move/strike diagonally")
		return 0

class Background:
	def __init__(self):
		self.world = {}
		self.ltile, self.ltile_rect = load_image('tile.png', 'terrain',-1)
		self.ltile = pygame.transform.scale(self.ltile,(globals.WIDTH,globals.HEIGHT))
		self.wtile, self.wtile_rect = load_image('tile-water1.png', 'terrain', -1)
		self.wtile = pygame.transform.scale(self.wtile,(globals.WIDTH,globals.HEIGHT))
		self.bgSurf = pygame.Surface((globals.WIDTH*10, globals.HEIGHT*10))
		self.__construct()
	
	def __construct(self):
		""" right now, this is hard-coded. eventually, I can read
		from a map file. not too hard """
		for y in range (0,10):
			for x in range (0,10):
				if (y in (4,5) and x in (2,3,6,7)): 
					self.bgSurf.blit(self.wtile, (x*globals.WIDTH, y*globals.HEIGHT))
					self.world[x,y] = constants.TERRAIN_IMPASSABLE
				else:
					self.bgSurf.blit(self.ltile, (x*globals.WIDTH, y*globals.HEIGHT))
					self.world[x,y] = constants.TERRAIN_NORMAL

	def getBackground(self):
		return self.bgSurf

	def terrainAt(self, (x,y)):
		try:
			return self.world[x, y]
		except KeyError:
			return None # or should this be constants.TERRAIN_IMPASSABLE ?
# -------------------------------------------
# M A I N
# -------------------------------------------
def main(argv):
	# ---------------------------------------------------------------------
	# handle command-line arguments
	# ---------------------------------------------------------------------
	try:
		opts, args = getopt.getopt(argv, \
		"sH:p:hC:r:f", \
		["server","host=","port=","help","color","resolution=","fullscreen"])
	except getopt.GetoptError:
		printUsage()
		sys.exit(2)
	host = "127.0.0.1"
	port = 9998
	server = 0
	for opt, arg in opts:
		if opt in ("-h","--help"):
			printUsage()
			sys.exit()
		elif opt in ("-s", "--server"):
			server = 1
		elif opt in ("-H", "--host"):
			host = arg
		elif opt in ("-p", "--port"):
			port = int(arg)
			print "using port: ", port
		elif opt in ("-C", "--color"):
			if arg.lower() == "red":
				globals.color = constants.RED
			elif arg.lower() == "blue":
				globals.color = constants.BLUE
				globals.myturn = 0 # red always goes first
			else:
				print "invalid color", arg
				sys.exit(0)
		elif opt in ("-r", "--resolution"):
			if arg in ("640x480", "800x600", "1024x768", "1280x1024"):
				vw, vh = arg.split("x")
				globals.VWIDTH, globals.VHEIGHT = int(vw), int(vh)
				if arg == "640x480":
					""" use at your own risk """
					globals.HEIGHT = 32
					globals.WIDTH  = 32
				if arg == "800x600":
					globals.HEIGHT = 48
					globals.WIDTH  = 48
				elif arg in ("1024x768", "1280x1024"):
					""" use full-size tiles"""
					globals.HEIGHT = 64
					globals.WIDTH  = 64
				else:
					pass
			else:
				print "invalid video mode, using default (1024x768)"
		elif opt in ("-f", "--fullscreen"):
			globals.fullscreen = 1
	
	# ---------------------------------------------------------------------
	# setup pygame and build the board, box and text objects/surfaces -- i.e. the game areas
	# ---------------------------------------------------------------------
	pygame.display.init()
	pygame.font.init() 
	if globals.fullscreen:
		screen = pygame.display.set_mode((globals.VWIDTH,globals.VHEIGHT), FULLSCREEN)
	else:
		screen = pygame.display.set_mode((globals.VWIDTH,globals.VHEIGHT))
	pygame.display.set_caption("Pytego")


	""" do I need to .convert() these? """
	boardSurface = pygame.Surface((globals.WIDTH*10, globals.HEIGHT*10))
	boardBGSurface = None # create and initialize below
	boxSurface = pygame.Surface((globals.WIDTH*4, globals.HEIGHT*10))
	boxBGSurface = pygame.Surface((globals.WIDTH*4, globals.HEIGHT*10))
	textSurface = pygame.Surface((screen.get_width(), (screen.get_height()-boardSurface.get_height())))

	""" do up the board and its background """
	board = GameArea((0,0,boardSurface.get_width(),boardSurface.get_height()), handleBoardEvent) 
	boardBG = Background()
	board.background = boardBG
	boardBGSurface = boardBG.getBackground()

	""" do up the box background, using fill() """
	boxBGSurface.fill((0,0,0))
	""" do up the box using sprites """
	box = GameArea((boardSurface.get_width(),0,boxSurface.get_width(), boxSurface.get_height()), handleBoxEvent) 
	col = 0
	row = 0
	for piece in constants.STARTING_PIECES:
		for i in range(0, constants.STARTING_PIECES[piece]):
			u = Unit(piece, globals.color, (col, row))
			box.addPiece(u)
			if col == 3:
				col = 0
				row = row + 1
			else:
				col = col + 1
	
	""" do up the text area """
	globals.textArea = TextArea((0,boardSurface.get_height(),textSurface.get_width(),textSurface.get_height()))

	""" do up the status area """
	globals.statusArea = StatusArea((boardSurface.get_width()+boxSurface.get_width(),0,boxSurface.get_width(), boxSurface.get_height()))

	# ---------------------------------------------------------------------
	# initialize networking
	# ---------------------------------------------------------------------
	connection = None
	sock = socket.socket (socket.AF_INET, socket.SOCK_STREAM)
	if server:
		try:
			sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
			sock.bind(('',port))
		except:
			print 'error binding socket, may already be in use'
			sys.exit(1)
		sock.listen(1)
		""" wait for client """
		newsock, addr = sock.accept()
		connection = Connection(socket=newsock)
	else:
		sock.connect ((host,port))
		connection = Connection(socket=sock, host=host, port=port)

	connthread = ConnectionPollerThread(connection)
	connthread.start()

	# ---------------------------------------------------------------------
	# Initialize game variables, do initial game setup
	# ---------------------------------------------------------------------
	done = 0
	read_net = 0
	unitSelected = 0
	taget = None
	selectedUnit = None
	onboard = 1

	# dirty_rect = None
	handler = None
	""" set allowable event types, first disable ALL """
	pygame.event.set_allowed(None)
	pygame.event.set_allowed(QUIT)
	pygame.event.set_allowed(KEYDOWN)
	pygame.event.set_allowed(MOUSEBUTTONDOWN)
	pygame.event.set_allowed(USEREVENT)

	globals.playing = constants.GAME_WAITING_TO_START

	globals.textArea.addText("Welcome to Pytego")
	if globals.color == constants.BLUE:
		globals.textArea.addText("You are Blue. Please place your pieces in the top four rows, then hit 'space' to begin.")
	else:
		globals.textArea.addText("You are Red. Please place your pieces in the bottom four rows, then hit 'space' to begin.")
	globals.textArea.addText("Good Luck!")
	# ---------------------------------------------------------------------
	# draw the initial game surfaces
	# ---------------------------------------------------------------------
	""" draw it up! """
	boardSurface.blit(boardBGSurface,(0,0))
	board.sprites.update()
	board.sprites.draw(boardSurface)

	boxSurface.blit(boxBGSurface,(0,0))
	box.sprites.update()
	box.sprites.draw(boxSurface)

	textSurface.blit(globals.textArea.getText(), (0,0))

	screen.blit(boardSurface, (board.rect.x, board.rect.y))
	screen.blit(boxSurface, (box.rect.x, box.rect.y))
	screen.blit(textSurface, (globals.textArea.rect.x, globals.textArea.rect.y))
	pygame.display.update()

	# ---------------------------------------------------------------------
	# begin main event loop
	# ---------------------------------------------------------------------
	while not done: 
		# ---------------------------------------------------------------------
		# get pygame and net-bourne events
		# ---------------------------------------------------------------------

		while read_net and globals.playing != constants.GAME_ENDED:
			if not checkNetEvents(connection):
				read_net = 0

		event = pygame.event.wait()

		if event.type is QUIT:
			done = 1
		if event.type is KEYDOWN:
			""" be careful, or this may turn inside out :( """
			if event.key is K_ESCAPE:
				done = 1
			elif event.key == K_SPACE:
				handleGameEvent(board, box, connection)
		if event.type is MOUSEBUTTONDOWN:
			try:
				whereclicked = [rect for rect in (board, box) if rect.rect.collidepoint(event.pos)][0]
				if not unitSelected:
					selectedUnit = whereclicked.unitAt(whereclicked.s2g(event.pos))
					handler = whereclicked.handleEvent
					target = None
					if selectedUnit:
						if selectedUnit.color == globals.color:
							unitSelected = 1
						else:
							globals.textArea.addText("try as you might, you may not select your opponent's unit")
				else:
					unitSelected = 0 # ??
					targetx, targety = whereclicked.s2g(event.pos)
					otherarea = [rect for rect in (board, box) if rect.rect.collidepoint(event.pos)][0]
					handler(otherarea, connection, selectedUnit, (targetx, targety))
			except IndexError:
				""" did not click on a game area """
				globals.textArea.addText("did not click on game area")
				pass

		if event.type is USEREVENT:
			if event.code == constants.USEREVENT_SOCKET_DATA:
				read_net = 1
			else:
				print "unknown event"

		# ---------------------------------------------------------------------
		# update the world 
		# ---------------------------------------------------------------------
		""" should I deal with game state changes here or above """
		updateGameState()

		for q in globals.action_queue[:]:
			myaction = globals.action_queue.pop(0)
			if myaction and myaction is not None:
				if myaction.action == action.MOVE:
					"""move the unit """
					# dirty_rect = pygame.Rect((myaction.unit.gx*globals.WIDTH, myaction.unit.gy*globals.HEIGHT, globals.WIDTH, globals.HEIGHT))
					board.movePiece(myaction.unit, myaction.gx, myaction.gy)
				elif myaction.action == action.REMOVE:
					""" remove the unit """
					# dirty_rect = pygame.Rect((myaction.unit.gx*globals.WIDTH, myaction.unit.gy*globals.HEIGHT, globals.WIDTH, globals.HEIGHT))
					globals.textArea.addText(myaction.unit.color + " " + myaction.unit.unit + " has been destroyed")
					globals.statusArea.update(myaction.unit)
					board.removePiece(myaction.unit)
					if globals.playing == constants.GAME_PLAYING and myaction.unit == 'flag':
						globals.playing = constants.GAME_ENDED
				elif myaction.action == action.PLACE:
					u = Unit(myaction.unit.unit, myaction.unit.color,(myaction.unit.gx, myaction.unit.gy))
					box.removePiece(myaction.unit)
					u.gx = myaction.gx
					u.gy = myaction.gy
					board.addPiece(u)
					board.movePiece(u, u.gx, u.gy)
				elif myaction.action == action.DEPLOY:
					u = Unit(myaction.unit.unit, myaction.unit.color,(myaction.unit.gx, myaction.unit.gy))
					board.addPiece(u)
					board.movePiece(u, u.gx, u.gy)
					globals.opponent_deployed = 1
				elif myaction.action == action.DONE:
					if globals.playing == constants.GAME_PLAYING:
						globals.myturn = 1
				elif myaction.action == action.VICTORY:
					globals.textArea.addText("You have failed. Better luck next time")
					globals.playing = constants.GAME_ENDED
				else:
					print "unknown action: ", myaction.action

		# ---------------------------------------------------------------------
		# update the screen
		# ---------------------------------------------------------------------
		board.sprites.update()
		box.sprites.update()

		boardSurface.blit(boardBGSurface,(0,0))
		board.sprites.draw(boardSurface)

		boxSurface.blit(boxBGSurface,(0,0))
		if globals.playing == constants.GAME_WAITING_TO_START:
			box.sprites.draw(boxSurface)
		else:
			boxSurface.blit(globals.statusArea.getStatus(), (0,0))

		textSurface.blit(globals.textArea.getText(), (0,0))

		screen.blit(boardSurface, (board.rect.x,board.rect.y))
		screen.blit(boxSurface, (box.rect.x,box.rect.y))
		screen.blit(textSurface, (globals.textArea.rect.x, globals.textArea.rect.y))
		pygame.display.update()

		# dirty_rect = None

	connection.close()
	sys.exit(1)

if __name__ == "__main__":
	main(sys.argv[1:])
